﻿using System;
using System.Diagnostics;
using System.Text.Json;
using System.Text.Json.Nodes;
using System.Text.Json.Serialization;

namespace HDLNoCGen
{
    /// <summary>
    /// Главный класс программы, содержащий точку входа и методы для управления проектами
    /// </summary>
    static class Program
    {
        /// <summary>
        /// Сбрасывает флаги метаданных проекта в JSON-файле на указанном этапе
        /// </summary>
        /// <param name="json_path">Путь к JSON-файлу с метаданными проекта</param>
        /// <param name="stage">Этап, до которого нужно сбросить флаги (0-3)</param>
        static void uncheckMetadata(string json_path, int stage)
        {
            string json = File.ReadAllText(json_path);
            ProjectSettings projectSettings = JsonSerializer.Deserialize<ProjectSettings>(json);

            if (stage <= 3)
            {
                projectSettings.databaseMetadata.writtenToDB = false;
            }
            if (stage <= 2)
            {
                projectSettings.quartusMetadata.quartusCompiled = false;
            }
            if (stage <= 1)
            {
                projectSettings.graphVerilogMetadata.verilogGenerated = false;
            }
            if (stage == 0)
            {
                projectSettings.graphVerilogMetadata.graphSerialized = false;
            }

            json = JsonSerializer.Serialize<ProjectSettings>(projectSettings, new JsonSerializerOptions { WriteIndented = true });
            File.WriteAllText(json_path, json);
        }

        /// <summary>
        /// Точка входа в программу. Обрабатывает аргументы командной строки и запускает соответствующие процессы.
        /// </summary>
        /// <param name="args">Аргументы командной строки</param>
        static void Main(string[] args)
        {
            // Инициализация процессов для управления проектом, генерации Verilog и компиляции в Quartus
            Process manager = new Process();
            Process veriloger = new Process();
            Process quartuser = new Process();
            Process dber = new Process();

            Process[] processes = { manager, veriloger, quartuser, dber }; 

            // Обработчик события завершения процесса для корректного завершения дочерних процессов
            AppDomain.CurrentDomain.ProcessExit += new EventHandler((sender, e) =>
            {
                foreach (Process process in processes)
                {
                    try
                    {
                        ProcessKiller.killProcessAndChildren(process.Id);
                    }
                    catch (InvalidOperationException)
                    {
                        // Процесс еще не был запущен
                    }
                }
            });

            // Флаги и параметры для управления проектом
            string key_arg = "";

            bool launch_manager = false;
            string project_name = "";
            string project_location = "";
            string project_new_name = "";
            string project_action = "o";

            // Флаги и параметры для генерации графа и Verilog
            bool launch_graph = false;
            string graph_args = "";

            // Флаги и параметры для компиляции в Quartus
            bool launch_quartus = false;
            string quartus_args = "";

            // Флаги и параметры для работы с базой данных
            bool launch_db = false;
            string db_args = "";

            bool launch_settings = false;
            string quartus_path = "";

            try
            {
                // Парсинг аргументов командной строки
                for (int i = 0; i < args.Length; i++)
                {
                    switch (args[i])
                    {
                        case "--project":
                            launch_manager = true;

                            bool project_breaker = false;
                            while (!project_breaker)
                            {
                                if (i + 1 < args.Length)
                                {
                                    switch (args[++i])
                                    {
                                        case "-n":
                                        case "--name":
                                            project_name = args[++i];
                                            break;
                                        case "-l":
                                        case "--location":
                                            project_location = args[++i];
                                            break;
                                        case "-o":
                                        case "--open":
                                            project_action = "o";
                                            break;
                                        case "-c":
                                        case "--create":
                                            project_action = "c";
                                            break;
                                        case "-e":
                                        case "--erase":
                                            project_action = "e";
                                            break;
                                        case "-r":
                                        case "--rename":
                                            project_action = "r";
                                            project_new_name = args[++i];
                                            break;
                                        default:
                                            i--;
                                            project_breaker = true;
                                            break;
                                    }
                                }
                                else
                                {
                                    project_breaker = true;
                                }
                            }
                            break;

                        case "--graph":
                            launch_graph = true;
                            key_arg = args[i];
                            break;
                        case "--quartus":
                            launch_quartus = true;
                            key_arg = args[i];
                            break;
                        case "--database":
                            launch_db = true;
                            key_arg = args[i];
                            break;
                        case "-h":
                        case "--help":
                            string help = File.ReadAllText("../../../help.txt");
                            Console.WriteLine(help);
                            return;
                        default:
                            if (key_arg == "")
                            {
                                Console.WriteLine($"Argument {args[i]} does not exist. Check the spelling or check graph generator numbers");
                                Environment.Exit(1);
                            }

                            switch (key_arg)
                            {
                                case "--graph":
                                    graph_args = $"{graph_args} {args[i]}";
                                    break;
                                case "--quartus":
                                    quartus_args = $"{quartus_args} {args[i]}";
                                    break;
                                case "--database":
                                    db_args = $"{db_args} {args[i]}";
                                    break;
                            }
                            break;
                    }
                }
            }
            catch (IndexOutOfRangeException e)
            {
                Console.WriteLine("No argument after an option that requires an argument");
                Environment.Exit(1);
            }

            // Запуск менеджера проектов
            if (launch_manager)
            {
                string manager_location = "../../../../../Project_manager/Project_manager/bin/Debug/net8.0";
                string manager_arguments = $"-l {project_location} -n {project_name} -{project_action} {(project_action == "r" ? project_new_name : "")}";

                manager.StartInfo.FileName = $"{manager_location}/Project_manager.exe";
                manager.StartInfo.Arguments = manager_arguments;
                manager.StartInfo.RedirectStandardOutput = true;
                manager.StartInfo.WorkingDirectory = manager_location;

                manager.OutputDataReceived += new DataReceivedEventHandler((sender, e) =>
                {
                    if (!String.IsNullOrEmpty(e.Data))
                    {
                        Console.WriteLine($"{e.Data}");
                    }
                });

                manager.Start();
                manager.BeginOutputReadLine();
                manager.WaitForExit();
                manager.Kill();

                if (project_action == "r")
                {
                    project_name = project_new_name;
                }

                if (manager.ExitCode != 0)
                {
                    Console.WriteLine("Project_manager failure");
                    Environment.Exit(1);
                }
                else
                {
                    Console.WriteLine("Project_manager success");
                }
            }

            // Запуск генератора графа и Verilog
            if (launch_graph)
            {
                uncheckMetadata($"{project_location}/{project_name}_metadata.json", 0);

                string veriloger_location = "../../../../../Graph_verilog_generator/Graph_verilog_generator/bin/Debug/net8.0";
                string veriloger_arguments = $"-l {project_location} -n {project_name} {graph_args}";

                veriloger.StartInfo.FileName = $"{veriloger_location}/Graph_verilog_generator.exe";
                veriloger.StartInfo.Arguments = veriloger_arguments;
                veriloger.StartInfo.RedirectStandardOutput = true;
                veriloger.StartInfo.WorkingDirectory = veriloger_location;

                veriloger.OutputDataReceived += new DataReceivedEventHandler((sender, e) =>
                {
                    if (!String.IsNullOrEmpty(e.Data))
                    {
                        Console.WriteLine($"{e.Data}");
                    }
                });

                veriloger.Start();
                veriloger.BeginOutputReadLine();
                veriloger.WaitForExit();
                veriloger.Kill();

                if (veriloger.ExitCode != 0)
                {
                    Console.WriteLine("Graph_verilog_generator failure");
                    Environment.Exit(1);
                }
                else
                {
                    Console.WriteLine("Graph_verilog_generator success");
                }
            }

            // Запуск компилятора Quartus
            if (launch_quartus)
            {
                uncheckMetadata($"{project_location}/{project_name}_metadata.json", 2);

                string quartuser_location = "../../../../../Quartus_compiler/Quartus_compiler/bin/Debug/net8.0";
                string quartuser_arguments = $"-l {project_location} -n {project_name}";

                quartuser.StartInfo.FileName = $"{quartuser_location}/Quartus_compiler.exe";
                quartuser.StartInfo.Arguments = quartuser_arguments;
                quartuser.StartInfo.RedirectStandardOutput = true;
                quartuser.StartInfo.WorkingDirectory = quartuser_location;

                quartuser.OutputDataReceived += new DataReceivedEventHandler((sender, e) =>
                {
                    if (!String.IsNullOrEmpty(e.Data))
                    {
                        Console.WriteLine($"{e.Data}");
                    }
                });

                quartuser.Start();
                quartuser.BeginOutputReadLine();
                quartuser.WaitForExit();
                quartuser.Kill();

                if (quartuser.ExitCode != 0)
                {
                    Console.WriteLine("Quartus_compiler failure");
                    Environment.Exit(1);
                }
                else
                {
                    Console.WriteLine("Quartus_compiler success");
                }
            }

            if (launch_db)
            {
                uncheckMetadata($"{project_location}/{project_name}_metadata.json", 3);

                string dber_location = "../../../../../Database_writer/Database_writer/bin/Debug/net8.0";
                string dber_arguments = $"-l {project_location} -n {project_name} {db_args}";

                dber.StartInfo.FileName = $"{dber_location}/Database_writer.exe";
                dber.StartInfo.Arguments = dber_arguments;
                dber.StartInfo.RedirectStandardOutput = true;
                dber.StartInfo.WorkingDirectory = dber_location;

                dber.OutputDataReceived += new DataReceivedEventHandler((sender, e) =>
                {
                    if (!String.IsNullOrEmpty(e.Data))
                    {
                        Console.WriteLine($"{e.Data}");
                    }
                });

                dber.Start();
                dber.BeginOutputReadLine();
                dber.WaitForExit();
                dber.Kill();

                if (dber.ExitCode != 0)
                {
                    Console.WriteLine("Database_writer failure");
                    Environment.Exit(1);
                }
                else
                {
                    Console.WriteLine("Database_writer success");
                }
            }
        }
    }
}